#include <FastLED.h>
#include "pitches.h"

#define DATA_PIN 3
#define NUM_LEDS 6
#define ROW 6
#define COL 5
CRGB leds[COL][NUM_LEDS];

const int potIn = A0;
boolean hasFailed = false; //check to see if the game finally lost
//Timing
int period = 1000;
int ledSpeed = 0;
unsigned long time_now = 0;
byte spawnNum = 0;
byte gamemode = 99;

byte ledPos[ROW][COL] = {
  {0, 0, 0, 0, 0},      //at Start(end of board)
  {0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0},     //at fingers
};

bool pressPeriod[6] = {false, false, false, false, false, false}; //last one decides if has clocked

int melody[] = {A6, FS6, G6, A6, FS6, G6, A6, A5, B5, CS6, D6, E6, FS6, G6, FS6, D6, E6, FS6, FS5, G5, A5, B5, A5, G5, A5, FS5, G5, A5, G5, B5, A5, G5, FS5, E5, FS5, E5, D5, E5, FS5, G5, A5, B5, G5, B5, A5, B5, CS6, D6, A5, B5, CS6, D6, E6, FS6, G6, A6};
int noteDurations[] = {8, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 8, 16, 16, 8, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 8, 16, 16, 8, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 8, 16, 16, 8, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16};

int mario[] = {EUP, EUP, EUP, C, EUP, GUP, G, C, G, E, A, A, A, A};

byte soundRound = 0;

int currentSpeedNum = 0;
bool hasSetNote = false;

void setup() {
  // put your setup code here, to run once:
  LEDS.addLeds<WS2812B, 2, GRB>(leds[0], NUM_LEDS); //1
  LEDS.addLeds<WS2812B, 3, GRB>(leds[1], NUM_LEDS); //2
  LEDS.addLeds<WS2812B, 4, GRB>(leds[2], NUM_LEDS); //3
  LEDS.addLeds<WS2812B, 5, GRB>(leds[3], NUM_LEDS); //4
  LEDS.addLeds<WS2812B, 6, GRB>(leds[4], NUM_LEDS); //5
  //pin A0 is the Speed Potentiometer
  pinMode(potIn, INPUT);  //Sound Potentiometer
  pinMode (13, OUTPUT);    //Speaker 
  pinMode(8, INPUT_PULLUP); //button 1
  pinMode(9, INPUT_PULLUP); //button 2
  pinMode(10, INPUT_PULLUP); //button 3
  pinMode(11, INPUT_PULLUP); // button 4
  pinMode(12, INPUT_PULLUP); // button 5
  randomSeed(analogRead(1)); //Completly randomize the outcome upon each bootup.
  Serial.begin(9600);
  Serial.println("Reset");
}

void loop() {
  // put your main code here, to run repeatedly:
  if (gamemode == 99) {
    if (digitalRead(8) == LOW) { //Normal
      playColor(0, 0, 100);
      gamemode = 0;
    }
    //      if (digitalRead(9) == LOW) { //Tetris
    //        playColor(0, 100, 100);
    //        gamemode = 0;
    //      }
    if (digitalRead(12) == LOW) { //FreeMode
      playColor(0, 100, 100);
      ledSpeed = analogRead(potIn);
      currentSpeedNum = map(ledSpeed, 0, 1023, 30, 4978);
      Serial.println(currentSpeedNum);
      gamemode = 1;
    }
  }
  if (gamemode == 0) {
    if (!hasFailed) {
      ledSpeed = analogRead(potIn);
      period = 1000 - map(ledSpeed, 0, 1023, 0, 850);
      checkButtons();
      playGame();
    } else {
      restart();
    }
  }
  if (gamemode == 1) {
    playGuitar();
  }
}
void checkButtons() {
  if (pressPeriod[5]) {
    pressPeriod[0] = false;
    pressPeriod[1] = false;
    pressPeriod[2] = false;
    pressPeriod[3] = false;
    pressPeriod[4] = false;
  }
  int noteDuration = 2500 / noteDurations[soundRound];
  if (digitalRead(8) == LOW) { //Pressing
    if (pressPeriod[0] == false) {
      if (ledPos[ROW - 1][0] == 1) {
        //continue
        ledPos[ROW - 1][0] = 0;
        tone(13, melody[soundRound], noteDuration);
        delay(period);
        soundRound++;
        pressPeriod[0] = true;
      } else {
        //fail
        Serial.println(ledPos[ROW - 1][0]);
        hasFailed = true;
      }
    }
  }
  if (digitalRead(9) == LOW) { //Pressing
    if (pressPeriod[1] == false) {
      if (ledPos[ROW - 1][1] == 1) {
        //continue
        ledPos[ROW - 1][1] = 0;
        tone(13, melody[soundRound], noteDuration);
        delay(period);
        pressPeriod[1] = true;
      } else {
        //fail
        hasFailed = true;
      }
    }
  }
  if (digitalRead(10) == LOW) { //Pressing
    if (pressPeriod[2] == false) {
      if (ledPos[ROW - 1][2] == 1) {
        //continue
        ledPos[ROW - 1][2] = 0;
        tone(13, melody[soundRound], noteDuration);
        delay(period);
        pressPeriod[2] = true;
      } else {
        //fail
        hasFailed = true;
      }
    }
  }
  if (digitalRead(11) == LOW) { //Pressing
    if (pressPeriod[3] == false) {
      if (ledPos[ROW - 1][3] == 1) {
        //continue
        ledPos[ROW - 1][3] = 0;
        tone(13, melody[soundRound], noteDuration);
        delay(period);
        pressPeriod[3] = true;
      } else {
        //fail
        hasFailed = true;
      }
    }
  }
  if (digitalRead(12) == LOW) { //Pressing
    if (pressPeriod[4] == false) {
      if (ledPos[ROW - 1][4] == 1) {
        //continue
        ledPos[ROW - 1][4] = 0;
        tone(13, melody[soundRound], noteDuration);
        delay(period);
        pressPeriod[4] = true;
      } else {
        //fail
        hasFailed = true;
      }
    }
  }
  pressPeriod[5] = false;
}
void playGame() {
  if (millis() > time_now + period) {
    Serial.println("Update");
    for (int i = 0; i < ROW; i++) {
      for (int j = 0; j < COL; j++) {
        Serial.print(ledPos[i][j]);
        if (ledPos[ROW - 1][j] == 1) {
          Serial.println("HAS PASSED LOSS");
          hasFailed = true;
          break;
        }
      }
      Serial.println("");
    }
    updateLEDS();
    addRandomLEDs();
  }
}
void updateLEDS() {
  for (int i = ROW - 1; i > 0; i--) {   // 7=6, 6=5, 5=4, 4=3, 3=2, 2=1, 1=0, 0 = Random val
    for (int j = 0; j < COL; j++) {
      ledPos[i][j] = ledPos[i - 1][j];
      ledPos[i - 1][j] = 0;
    }
  }
  for (int i = 0; i < ROW; i++) {
    for (int j = 0; j < COL; j++) {
      if (ledPos[i][j] == 0) {
        leds[j][i].setRGB(0, 0, 0); //delete previous led
      } else {
        leds[j][i].setHSV(j * 5 * 5 * 2, 255, 255); //switch led to next co
      }
      FastLED[j].showLeds();
    }
  }
  pressPeriod[5] = true;
  spawnNum = 0;
  time_now = millis();
}
void addRandomLEDs() {
  if (random8(8) == 1) {
    ledPos[0][0] = 1;
    leds[0][0].setRGB(255, 0, 0);
    FastLED[0].showLeds();
    spawnNum++;
  }
  if (random8(8) == 2 && spawnNum < 1) {
    ledPos[0][1] = 1;
    leds[1][0].setRGB(255, 255, 0);
    FastLED[1].showLeds();
    spawnNum++;
  }
  if (random8(8) == 3 && spawnNum < 1) {
    ledPos[0][2] = 1;
    leds[2][0].setRGB(0, 255, 0);
    FastLED[2].showLeds();
    spawnNum++;
  }
  if (random8(8) == 4 && spawnNum < 1) {
    ledPos[0][3] = 1;
    leds[3][0].setRGB(0, 0, 255);
    FastLED[3].showLeds();
    spawnNum++;
  }
  if (random8(8) == 5 && spawnNum < 1) {
    ledPos[0][4] = 1;
    leds[4][0].setRGB(255, 0, 255);
    FastLED[4].showLeds();
    spawnNum++;
  }
}
void restart() {
  playColor(100, 0, 0);
  soundRound = 0;
  pressPeriod[0] = false;
  pressPeriod[1] = false;
  pressPeriod[2] = false;
  pressPeriod[3] = false;
  pressPeriod[4] = false;
  pressPeriod[5] = false;
  hasFailed = false;
  gamemode = 99;
}
void playColor(byte r, byte g, byte b) {
  for (int i = 0; i < ROW; i++) {
    for (int j = 0; j < COL; j++) {
      ledPos[i][j] = 0;
      leds[j][i].setRGB(r, g, b);
      FastLED[j].showLeds();
      delay(40);
    }
  }
  for (int i = 0; i < COL; i++) {
    for (int j = 0; j < ROW; j++) {
      leds[i][j].setRGB(0, 0, 0);
      FastLED[i].showLeds();
      delay(40);
    }
  }
}
int pitch[5] = {C2, E7, G5, D7, CS8};
int duration = 2500 / noteDurations[8];
void playGuitar() {
  if (millis() > time_now + 500) {
    time_now = millis();
    ledSpeed = analogRead(potIn);
    int readPitch = map(ledSpeed, 0, 1023, 30, 4978);
    if (currentSpeedNum != readPitch && !hasSetNote) {
      currentSpeedNum = readPitch;
      hasSetNote = true;
    }
  }
  if (digitalRead(8) == LOW) { //Pressing
    if(hasSetNote){
      pitch[0] = currentSpeedNum;
      hasSetNote = false;
    }
    tone(13, pitch[0], duration);
    for (int i = 1; i <= ROW; i++) {
      leds[0][NUM_LEDS - i].setRGB(255, 0, 0);
      FastLED[0].showLeds();
      delay(10);
    }
    delay(100);
  }
  if (digitalRead(9) == LOW) { //Pressing
    if(hasSetNote){
      pitch[1] = currentSpeedNum;
      hasSetNote = false;
    }
    tone(13, pitch[1], duration);
    for (int i = 1; i <= ROW; i++) {
      leds[1][NUM_LEDS - i].setRGB(255, 225, 0);
      FastLED[1].showLeds();
      delay(10);
    }
    delay(100);
  }
  if (digitalRead(10) == LOW) { //Pressing
     if(hasSetNote){
      pitch[2] = currentSpeedNum;
      hasSetNote = false;
    }
    tone(13, pitch[2], duration);
    for (int i = 1; i <= ROW; i++) {
      leds[2][NUM_LEDS - i].setRGB(0, 255, 0);
      FastLED[2].showLeds();
      delay(10);
    }
    delay(100);
  }
  if (digitalRead(11) == LOW) { //Pressing
        if(hasSetNote){
      pitch[3] = currentSpeedNum;
      hasSetNote = false;
    }
    tone(13, pitch[3], duration);
    for (int i = 1; i <= ROW; i++) {
      leds[3][NUM_LEDS - i].setRGB(0, 0, 255);
      FastLED[3].showLeds();
      delay(10);
    }
    delay(100);
  }
  if (digitalRead(12) == LOW) { //Pressing
        if(hasSetNote){
      pitch[4] = currentSpeedNum;
      hasSetNote = false;
    }
    tone(13, pitch[4], duration);
    for (int i = 1; i <= ROW; i++) {
      leds[4][NUM_LEDS - i].setRGB(255, 0, 255);
      FastLED[4].showLeds();
      delay(10);
    }
    delay(100);
  }
  for (int i = 1; i <= ROW; i++) {
    leds[0][NUM_LEDS - i].setRGB(0, 0, 0);
    leds[1][NUM_LEDS - i].setRGB(0, 0, 0);
    leds[2][NUM_LEDS - i].setRGB(0, 0, 0);
    leds[3][NUM_LEDS - i].setRGB(0, 0, 0);
    leds[4][NUM_LEDS - i].setRGB(0, 0, 0);
    FastLED.show();
  }
  if (digitalRead(8) == LOW && digitalRead(9) == LOW && digitalRead(10) == LOW && digitalRead(11) == LOW && digitalRead(12) == LOW) { //Pressing
    restart();
  }
}
